package com.dzeio.openhealth.ui

import android.app.ActivityManager
import android.app.NotificationChannel
import android.app.NotificationManager
import android.content.Context
import android.content.Intent
import android.os.Build
import android.os.Bundle
import android.util.Log
import android.view.LayoutInflater
import android.view.Menu
import android.view.MenuItem
import androidx.navigation.NavController
import androidx.navigation.fragment.NavHostFragment
import androidx.navigation.ui.AppBarConfiguration
import androidx.navigation.ui.NavigationUI
import androidx.navigation.ui.navigateUp
import androidx.navigation.ui.setupActionBarWithNavController
import androidx.navigation.ui.setupWithNavController
import com.dzeio.openhealth.Application
import com.dzeio.openhealth.R
import com.dzeio.openhealth.core.BaseActivity
import com.dzeio.openhealth.databinding.ActivityMainBinding
import com.dzeio.openhealth.interfaces.NotificationChannels
import com.dzeio.openhealth.services.OpenHealthService
import com.dzeio.openhealth.workers.WaterReminderWorker
import dagger.hilt.android.AndroidEntryPoint

@AndroidEntryPoint
class MainActivity : BaseActivity<ActivityMainBinding>() {

    companion object {
        const val TAG = "${Application.TAG}/MainActivity"
    }

    private lateinit var appBarConfiguration: AppBarConfiguration

    private lateinit var navController: NavController

    override val bindingInflater: (LayoutInflater) -> ActivityMainBinding =
        ActivityMainBinding::inflate

    override fun onCreate(savedInstanceState: Bundle?) {
        setTheme(R.style.Theme_OpenHealth_NoActionBar)
        super.onCreate(savedInstanceState)

    }

    override fun onCreated(savedInstanceState: Bundle?) {
        super.onCreated(savedInstanceState)

        setSupportActionBar(binding.toolbar)

        val navHostFragment =
            supportFragmentManager.findFragmentById(R.id.nav_host_fragment) as NavHostFragment

        navController = navHostFragment.navController

        appBarConfiguration = AppBarConfiguration(
            setOf(
                R.id.nav_home,
                R.id.nav_browse,
                R.id.nav_activity,
                R.id.nav_extensions
            )
        )

        setupActionBarWithNavController(navController, appBarConfiguration)

        binding.bottomNav.setupWithNavController(navController)

//        registerForActivityResult(ActivityResultContracts.RequestPermission()) {
//
//        }
//            .launch(Manifest.permission.ACTIVITY_RECOGNITION)

        createNotificationChannel()

        // Services
        WaterReminderWorker.setup(this)
//        StepCountService.setup(this)

        this.betterStartService(OpenHealthService::class.java)
    }

    private fun <T> betterStartService(service: Class<T>) {
        val activityManager = getSystemService(Context.ACTIVITY_SERVICE) as ActivityManager
        for (runninService in activityManager.getRunningServices(Integer.MAX_VALUE)) {
            if (service.name.equals(runninService.service.className)) {
                Log.w(TAG, "Service already existing, not starting again")
                return
            }
        }
        Log.i(TAG, "Starting service ${service.name}")
        Intent(this, service).also { intent -> startService(intent) }
    }

    override fun onCreateOptionsMenu(menu: Menu): Boolean {
        super.onCreateOptionsMenu(menu)
        menuInflater.inflate(R.menu.main, menu)
        return true
    }

    override fun onSupportNavigateUp(): Boolean =
        navController.navigateUp(appBarConfiguration) || super.onSupportNavigateUp()

    override fun onOptionsItemSelected(item: MenuItem): Boolean =
        NavigationUI.onNavDestinationSelected(item, navController) ||
            super.onOptionsItemSelected(item)

    @Deprecated("Deprecated in Java")
    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        Log.d("MainActivity", "onActivityResult $requestCode $resultCode")
        for (fragment in supportFragmentManager.primaryNavigationFragment!!.childFragmentManager.fragments) {
            fragment.onActivityResult(requestCode, resultCode, data)
        }
    }

    private fun createNotificationChannel() {
        val notificationManager: NotificationManager =
            getSystemService(Context.NOTIFICATION_SERVICE) as NotificationManager

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            for (channel in NotificationChannels.values()) {
                Log.d("MainActivity", channel.channelName)
                try {
                    notificationManager.createNotificationChannel(
                        NotificationChannel(
                            channel.id,
                            channel.channelName,
                            channel.importance
                        )
                    )
                } catch (e: Exception) {
                    Log.e("MainActivity", "Error Creating Notification Channel", e)
                }
            }
        }
    }
}
